<?php

namespace Bixo\Document\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Document\Http\Requests\DocumentsRequest;
use Bixo\Document\Interfaces\DocumentsRepositoryInterface;
use Bixo\Document\Models\Documents;
use Bixo\Document\Forms\Documents as Form;

/**
 * APIController  class for documents.
 */
class DocumentsAPIController extends BaseController
{

    /**
     * Initialize documents resource controller.
     *
     * @param type DocumentsRepositoryInterface $documents
     *
     * @return null
     */
    public function __construct(DocumentsRepositoryInterface $documents)
    {
        parent::__construct();
        $this->repository = $documents;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Document\Repositories\Criteria\DocumentsResourceCriteria::class);
    }

    /**
     * Display a list of documents.
     *
     * @return Response
     */
    public function index(DocumentsRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Document\Repositories\Presenter\DocumentsPresenter::class)
            ->paginate();
    }

    /**
     * Display documents.
     *
     * @param Request $request
     * @param Model   $documents
     *
     * @return Response
     */
    public function show(DocumentsRequest $request, Documents $documents)
    {
        return $documents->setPresenter(\Bixo\Document\Repositories\Presenter\DocumentsListPresenter::class);
        ;
    }

    /**
     * Create new documents.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DocumentsRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('document::documents.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('document/documents/' . $documents->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('document/documents');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the documents.
     *
     * @param Request $request
     * @param Model   $documents
     *
     * @return Response
     */
    public function update(DocumentsRequest $request, Documents $documents)
    {
        try {
            $data = $request->all();

            $documents->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('document::documents.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('document/documents/' . $documents->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('document/documents/' . $documents->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the documents.
     *
     * @param Model   $documents
     *
     * @return Response
     */
    public function destroy(DocumentsRequest $request, Documents $documents)
    {
        try {
            $documents->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('document::documents.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('document/documents/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('document/documents/' . $documents->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
